/*
 *	Copyright (C) 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */


#include <QtGui>

#include "link.h"
#include "node.h"

Link::Link(Node *fromNode, QString fromProp, Node *toNode, QString toProp, QGraphicsItem * parent, QGraphicsScene * scene): QGraphicsLineItem(parent, scene)
{
    myFromNode = fromNode;
    myToNode = toNode;
	myFromProp = fromProp;
	myToProp = toProp;

    myFromNode->addOutLink(this);
    myToNode->addInLink(this);

    setFlags(QGraphicsItem::ItemIsSelectable);
    setZValue((myFromNode->zValue() > myToNode->zValue()) ? myFromNode->zValue()+1 : myToNode->zValue()+1);

    trackNodes();
}

Link::~Link()
{
    myFromNode->removeLink(this);
    myToNode->removeLink(this);
}

Node *Link::fromNode() const
{
    return myFromNode;
}

Node *Link::toNode() const
{
    return myToNode;
}

QString Link::fromProp() const
{
	return myFromProp;
}

QString Link::toProp() const
{
	return myToProp;
}

void Link::setColor(const QColor &color)
{
    setPen(QPen(color, 1.0));
}

QColor Link::color() const
{
    return pen().color();
}


void Link::trackNodes()
{
	// actualiza el zValue al máximo de los dos, o al del mayor grupo al que pertenecen
	// reducimos el tamaño del Pen del link en función del número de antecesores (*0.5 por cada uno)
	//(solo se mira para uno pq no se permiten enlaces entre grupos)
	setZValue((myFromNode->zValue() > myToNode->zValue()) ? myFromNode->zValue() + 1 : myToNode->zValue() + 1);

	QGraphicsItem *ancestor = myFromNode->parentItem();
	QPen myPen = pen();
    while (ancestor) {
		setZValue(ancestor->zValue() + 1);
		myPen.setWidthF(myPen.width() * 0.4);
		ancestor = ancestor->parentItem();
	}
	setPen(myPen);


	setLine( QLineF(myFromNode->scenePointPos(myFromProp, FALSE), myToNode->scenePointPos(myToProp, TRUE)) );
}

