/*
 *	Copyright (C) 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef NODE_H
#define NODE_H

#include <QApplication>
#include <QColor>
#include <QGraphicsItem>
#include <QSet>

#include "link.h"

#ifndef DOXYGEN_IGNORE_THIS

class Node : public QGraphicsItem
{
    Q_DECLARE_TR_FUNCTIONS(Node)

public:
    Node(QGraphicsItem * parent = 0, QGraphicsScene * scene = 0);

    void setText(const QString &text);
    QString text() const;
    void setTextColor(const QColor &color);
    QColor textColor() const;
    void setOutlineColor(const QColor &color);
    QColor outlineColor() const;
    void setBackgroundColor(const QColor &color);
    QColor backgroundColor() const;

    void addInLink(Link *link);
    void addOutLink(Link *link);
    void removeLink(Link *link);
	QList<Link *> getLinks() const;
	QList<Link *> getInLinks() const;
	QList<Link *> getOutLinks() const;
	int precursors(QList<Node *> tail = QList<Node *>());

	QString getName() const { return name; }
	QString getType() const { return type; }
	void setName(QString _name) { name = _name; }
	void setType(QString _type) { type = _type; }
	virtual QString propName(int point) const = 0; // devuelve el nombre de la propiedad señalada por point

    QRectF boundingRect() const;
    QPainterPath shape() const;
    virtual void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) = 0;
	virtual QPointF scenePointPos(int point) const = 0;
	virtual QPointF scenePointPos(QString name, bool in) const = 0;
	virtual void updateLinksPos() = 0;

protected:
    virtual QRectF outlineRect() const = 0;
    virtual int roundness(double size) const = 0;
	virtual int pointAt(QPointF pos) const = 0;
	void mouseMoveEvent(QGraphicsSceneMouseEvent *event);

    QList<Link *> myInLinks;
	QList<Link *> myOutLinks;

    QString myText;
    QColor myTextColor;
    QColor myBackgroundColor;
    QColor myOutlineColor;

	QString name;
	QString type;
};

#endif
#endif
