/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvio.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvio.h>
#include <qvipp.h>

// Writes the raw content of an image to a file.
template<int Channels> void readUCharImageFileFromImageData(QFile &file, QVImage<uChar, Channels> &image)
	{
	for (uInt i = 0; i < image.getRows(); i++)
		file.read((char *) image.getWriteData() + i*image.getStep(), Channels * image.getCols());
	}

template<int Channels> void writeUCharImageDataToImageFile(QFile &file, const QVImage<uChar, Channels> &image)
	{
	for (uInt i = 0; i < image.getRows(); i++)
		file.write((char *) image.getReadData() + i*image.getStep(), Channels * image.getCols());
	}

bool writeYUV4MPEG2Header(QFile &file, const int cols, const int rows, const int fps)
	{
	file.write(QString("YUV4MPEG2 W%1 H%2 F%3:1 Ip A0:0\x0a").arg(cols).arg(rows).arg(fps).toAscii());

	return true;
	}

bool writeYUV4MPEG2Frame(QFile &file, const QVImage<uChar,1> imageY, const QVImage<uChar,1> imageU, const QVImage<uChar,1> imageV)
	{
	// Write 'FRAME' keyword.
	file.write(QString("FRAME\x0a").toAscii());

	// Write image data buffers
	writeUCharImageDataToImageFile<1>(file, imageY);
	writeUCharImageDataToImageFile<1>(file, imageU);
	writeUCharImageDataToImageFile<1>(file, imageV);

	file.flush();

	return true;
	}

bool writeYUV4MPEG2Frame(QFile &file, const QVImage<uChar,3> imageRGB)
	{
	const int cols = imageRGB.getCols(), rows = imageRGB.getRows();
	QVImage<uChar, 1> imageY(cols, rows);
	QVImage<uChar, 1> imageU(cols/2, rows/2);
	QVImage<uChar, 1> imageV(cols/2, rows/2);

	RGBToYUV420(imageRGB, imageY, imageU, imageV);

	writeYUV4MPEG2Frame(file, imageY, imageU, imageV);

	return true;
	}

bool writeQVImageToFile(const QString fileName, const QVImage<uChar, 3> &image)
	{ return ((QImage) image).save(fileName); }

bool readQVImageFromFile(const QString fileName, QVImage<uChar, 3> &image)
	{
	QImage qimg;

	if(not qimg.load(fileName)) return FALSE;
	
	image = QVImage<uChar, 3>(qimg);

	return TRUE;
	}
